/* 
 * MAT259 Project 3: Data correlation
 * Kitty Currier
 * currier@geog.ucsb.edu
 * 25 Feb. 2014
 *
 * This code must be used in conjunction
 * with wc_setup files, which perform the 
 * NYT query and generates JSON and GIF 
 * images used by this code.
 */

PFont arial12 = createFont("Arial", 12, true);
PFont arialb16 = createFont("Arial Bold", 16, true);

int screenW = 740;
int screenH = 580;
int rectW = 50;
int rectH = 20;
int xOffset = 70;

String[] months = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

int[] splCount = new int[12];
int[] nytCount = new int[12];
int splTotal = 0;
int nytTotal = 0;
int splMax = 0;
int nytMax = 0;
int splMin = 100000;  // arbitrarily large number
int nytMin = 100000; 
PImage[] wcSPL = new PImage[12];
PImage[] wcNYT = new PImage[12];

int mouseBox = 0;

color[] rgbGreens = { color(247,252,253), // lowest
                      color(229,245,249), 
                      color(204,236,230), 
                      color(153,216,201), 
                      color(102,194,164), 
                      color(65,174,118),
                      color(35,139,69),
                      color(0,109,44), 
                      color(0,68,27)  };  // highest
                      
color[] rgbBlues = {  color(247,251,255),
                      color(222,235,247),
                      color(198,219,239),
                      color(158,202,225),
                      color(107,174,214),
                      color(66,146,198),
                      color(33,113,181),
                      color(8,81,156),
                      color(8,48,107)  }; 
                      
void setup() {
  size(screenW, screenH);
  colorMode(RGB);
  smooth();
  loadData();
}


void draw() {
  background(255);
   
  // list of months
  noStroke();
  textFont(arial12);
  fill(0);
  textAlign(CENTER, CENTER);
  for(int i=0; i<12; i++) {
    text(months[i], i*rectW+xOffset+rectW/2, screenH/2);
  }
  
  
  // SPL data bar
  stroke(255); 
  strokeWeight(1);
  for(int i=0; i<12; i++) {
    int value;
    float interval = splMax/8;
    value = floor(splCount[i]/interval);
    fill(rgbGreens[value]);
    rect(i*50+xOffset, screenH/2-30, rectW, rectH);
  } 
  
  // NYT data bar
  for(int i=0; i<12; i++) {
    int value;
    float interval = log(nytMax)/8;
    value = floor(log(nytCount[i])/interval);
    fill(rgbBlues[value]);
    rect(i*50+xOffset, screenH/2+15, rectW, rectH);
  }
  
  // wc images
  PImage imgSPL = new PImage();
  PImage imgNYT = new PImage();
  imgSPL = wcSPL[mouseBox];
  imgNYT = wcNYT[mouseBox];
  image(imgSPL, 20, 10, imgSPL.width, imgSPL.height);
  image(imgNYT, 20, screenH/2+rectH+45, imgNYT.width, imgNYT.height);
  
  //olympic rings
  PImage rings = new PImage();
  rings = loadImage("rings.gif");
  image(rings, xOffset+rectW*7, screenH/2-55, rectW, rings.height/2);
  
  // hilight box
  pushMatrix();
  translate(mouseBox*rectW, 0);
  stroke(0);
  strokeWeight(3);
  noFill();
  rect(xOffset, screenH/2-30, rectW, 3*rectH+4);
  popMatrix();
  
  // SPL & NYT names
  textAlign(LEFT, CENTER);
  pushMatrix();
  translate(40,screenH/2-1.5*rectH);
  rotate(radians(-90));
  fill(155);
  textFont(arialb16);
  text("Seattle Public Library", 0, 0);
  popMatrix();
  
  pushMatrix();
  textAlign(LEFT, CENTER);
  translate(screenW-20, screenH/2+135);
  rotate(radians(90));
  text("The New York Times", 190-screenH/2, 20);
  popMatrix();
  
  if(keyPressed) {
    saveFrame("image.png"); 
  }
}

void loadData() {
  JSONObject manifest = loadJSONObject("../wc_setup/data/manifest.json");
    
  for(int i=0; i<12; i++) {
    splCount[i] = manifest.getJSONArray("Counts by month").getJSONObject(i).getInt("SPL count");
    nytCount[i] = manifest.getJSONArray("Counts by month").getJSONObject(i).getInt("NYT count");
    splTotal += splCount[i];
    nytTotal += nytCount[i];   
    if(splCount[i]>splMax) {
      splMax = splCount[i];
    } else if(splCount[i]<splMin) {
      splMin = splCount[i];
    }
    if(nytCount[i]>nytMax) {
      nytMax = nytCount[i];
    } else if(nytCount[i]<nytMin) {
      nytMin = nytCount[i];
    }
//    println("month: "+i);
//    println("SPL count: "+splCount[i]);
//    println("NYT count: "+nytCount[i]);
  } 
  println("SPL total: "+splTotal);
  println("SPL max: "+splMax);
  println("SPL min: "+splMin);
  println("NYT total: "+nytTotal);
  println("NYT max: "+nytMax);
  println("NYT min: "+nytMin);
  
  // load word cloud images
  StringBuffer sb1;
  StringBuffer sb2;
  String filepath1;
  String filepath2;
  
  for(int i=0; i<12; i++) {
    sb1 = new StringBuffer();
    sb1.append("../wc_setup/data/spl");
    sb1.append(i);
    sb1.append(".gif");
    filepath1 = sb1.toString();
    wcSPL[i] = loadImage(filepath1);
    sb2 = new StringBuffer();
    sb2.append("../wc_setup/data/nyt");
    sb2.append(i);
    sb2.append(".gif");
    filepath2 = sb2.toString();
    wcNYT[i] = loadImage(filepath2);
  } 
  
}
