/* This code is for downloading and parsing
 * the results of a query w/the NYT Article 
 * Search API. The output is formatted to work 
 * with the WordCram library.
 * Original code from Joshua Dickinson & Yoon Chung Han.
 * Works only in Processing v2.x
 */

// **Results to use with WC and viz**
String[] nytByMonth = new String[12]; // for storing input for WC by month
int[] nytCountByMonth = new int[12]; // counts titles by month

JSONArray entryArr = new JSONArray(); // for storing NYT and SPL counts by month

String baseURL = "http://api.nytimes.com/svc/search/v2/articlesearch";
String apiKey = "c9b4a5278dfe3aa89d66972eef4f847b:13:65477885"; // Kitty's Article Search API key
String[][] datesTitlesAbs; // for storing dates, headlines and abstracts returned from query
int count = 0; // for article count
int counter = 0; // iterator for filling datesTitlesAbs
int pages; // no. of pages of results returned by NYT API

String[][] getArticleDatesTitles(String word, String beginDate, String endDate) {
  
  if(newQuery) {
    int pages = 1; // pages of results returned; this value
    try {
      String filename = new String();
      for(int page=0; page<pages; page++) {
        StringBuffer buff = new StringBuffer();
        buff.append("data/nytDocsArray"+page+".json");
        filename = buff.toString();
      
        Thread.sleep(100); // to avoid exceeding QPS
        String request = "";
        request += baseURL + ".json";
        request += "?fq=headline:" + nytWord; // filter query to only return items with this word in headline
        request += "&fl=headline,pub_date,abstract"; // only return the "headline", "pub_date" and "abstract" fields
        request += "&sort=oldest"; // oldest first
        request += "&page="+page; // specifies page of results to return
        request += "&begin_date=" + beginDate;
        request += "&end_date=" + endDate;
        request += "&api-key=" + apiKey;
     
        println(request);
     
        JSONObject root = loadJSONObject(request);
        JSONObject response = root.getJSONObject("response");
        JSONArray docsA = response.getJSONArray("docs");
        
        saveJSONArray(docsA,filename);
       
        if(page==0) { //only do this on the first iteration
          count = response.getJSONObject("meta").getInt("hits");
          manifest.setInt("NYT total count",count);
          datesTitlesAbs = new String[count][3]; // initialize array for dates/titles/abstract
          println("Article count: "+count);
          if(count>10) {
            pages = floor(count/10);
            if(count%10>0) {
              pages++; 
            }
            println("Pages of results: "+pages);
            manifest.setInt("NYT pages",pages);
          }
        }
        datesTitlesAbs = addInfo(docsA, datesTitlesAbs);  
      } // end of for loop over pages
      
    } catch (Exception e) {
      e.printStackTrace();
    }
    
    return datesTitlesAbs;  
    
  } else { // if newQuery==false, use cached files
    JSONArray jsonArr = new JSONArray();
//    manifest = loadJSONObject("data/manifest.json");
    int fileCount = manifest.getInt("NYT pages");
    int count = manifest.getInt("NYT total count");
    String filename = new String();
    datesTitlesAbs = new String[count][3]; // initialize array for dates/titles/abstract
    
    for(int i = 0; i<fileCount; i++) {
       StringBuffer buff = new StringBuffer();
       buff.append("data/nytDocsArray"+i+".json");
       filename = buff.toString();
       jsonArr = loadJSONArray(filename);
       datesTitlesAbs = addInfo(jsonArr, datesTitlesAbs);      
    }
    return datesTitlesAbs;
  }
}
  

// helper fxn for getArticleDatesTitles
String[][] addInfo(JSONArray jArray, String[][] sArray) {
  JSONArray jsonArr = jArray;
  String[][] container = sArray;
  
  // add main title and kicker to stringbuffer, 
  // then add pub_date and strinbuffer titles and abstract
  // to datesTitlesAbs
  for(int i=0; i<jsonArr.size(); i++) {
    String pubDate = jsonArr.getJSONObject(i).getString("pub_date");
    StringBuffer buff = new StringBuffer(); // buff is cleared every time
    buff.append(" ");
    buff.append(jsonArr.getJSONObject(i).getJSONObject("headline").getString("main"));
    buff.append(" ");
    if(jsonArr.getJSONObject(i).getJSONObject("headline").hasKey("kicker")) {
      buff.append(jsonArr.getJSONObject(i).getJSONObject("headline").getString("kicker")); // add main title and kicker to stringbuffer
      buff.append(" ");  
  }
  // check if the entry has an abstract, 
  // and that the abstract is not null
  if(jsonArr.getJSONObject(i).hasKey("abstract") && !jsonArr.getJSONObject(i).isNull("abstract")) {
    datesTitlesAbs[counter][2] = jsonArr.getJSONObject(i).getString("abstract"); 
  }
  datesTitlesAbs[counter][0] = pubDate;
  datesTitlesAbs[counter][1] = buff.toString();
  println("datesTitlesAbs["+counter+"]: "+datesTitlesAbs[counter][0]);
  println("datesTitlesAbs["+counter+"]: "+datesTitlesAbs[counter][1]);
  println("datesTitlesAbs["+counter+"]: "+datesTitlesAbs[counter][2]);
  counter++;
 }
 return container; 
}

String[] formatNYTtitles(String[][] input) {
  String[][] in = input;
  String[] out = new String[12]; // each string holds each month's text
  StringList[] slist = new StringList[12];
  String[] months = {"01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"};
  
  //initialize StringList objects
  for(int i=0; i<12; i++) {
   slist[i] = new StringList(); 
  }
  
  for(int i=0; i<in.length; i++) {
    for(int j=0; j<12; j++) {
      if(in[i][0].regionMatches(5,months[j],0,2)) { // if pub month matches current month, add it
        nytCountByMonth[j] ++;
        slist[j].append(in[i][1]);
        slist[j].append(in[i][2]); // append both "headline" and "abstract" fields
        break;
      }
    }
  }
  for(int i=0; i<12; i++) {
    StringBuffer buff = new StringBuffer();
    for(int j=0; j<slist[i].size(); j++){
      buff.append(slist[i].get(j));
    }
    out[i] = buff.toString();
  }
  println("**Article count by month**");
  for(int i=0; i<12; i++) {
    println(i+1+": "+nytCountByMonth[i]); 
  }
  saveManifest();
  return out;
}


void saveManifest() {
  // save a JSON file w/num of JSON files saved & article count
  for(int i=0; i<12; i++){
    JSONObject entry = new JSONObject();
    entry.setInt("NYT count", nytCountByMonth[i]);
    entry.setInt("SPL count", splCountByMonth[i]);
    entry.setInt("month",i);
    entryArr.setJSONObject(i, entry);
    println("manifest data: "+entryArr.getJSONObject(i));
  }
  manifest.setJSONArray("Counts by month", entryArr);
  saveJSONObject(manifest, "data/manifest.json");
}
