/*
 * Run this file to generate new word clouds
 * and/or perform new NYT query.
 *
 * WordCram library for generating word clouds 
 * from https://code.google.com/p/wordcram/
*/
import wordcram.*;

// * Set source: if newQuery == true, will
// * perform new NYT web query & write new files
// * to cache. If false, will use cached JSON 
// * files (but will generate new word cloud images).
boolean newQuery = false;

// for storing info to pass to other PDE files
JSONObject manifest = new JSONObject();

// **Set these parameters for NYT query**
String nytWord = "olympic?"; // the "?" is a wildcard--this must be a single term to work in query
String start = "20080101"; // start search on Jan. 1, 2008
String end = "20081231";  // end search on Dec. 31, 2008

PFont georgia = createFont("Georgia", 1);
WordCram wc; 
PImage cachedImage; // Cache the rendered wordcram, so drawing is fast
Word lastClickedWord; // The word that was under the user's last click
//String[] titleWords = new String[12]; // to collect titles for each month

void setup() {
  size(700,220);
  background(255);
  smooth();
  
  //load manifest if using cached files
  if(!newQuery) {
    manifest = loadJSONObject("data/manifest.json"); 
  }  

  // SPL data
  // import data from text file and parse Table into String Array
  // this must happen before NYT search so manifest updates correctly
  Table tableData = new Table("olympic2008.csv");
  parseTable(tableData, titleWords);
   
  //NYT data
  datesTitlesAbs = getArticleDatesTitles(nytWord, start, end);
  println("Count of articles: "+count);
  nytByMonth = formatNYTtitles(datesTitlesAbs); 
 
/*
 * Make NYT wordcrams from nytByMonth 
 */ 
 
  for(int i = 0; i<12; i++) {
    String filename = new String();
    StringBuffer buff = new StringBuffer();
    buff.append("data/nyt"+i+".gif");
    filename = buff.toString();
        
    wc = new WordCram(this).fromTextString(nytByMonth[i])
    .withStopWords("olympic olympics null") //words to exclude from count
    .withColorer(darkerByWeight(8,81,156))
    .angledAt(radians(0))
    .withFont("Georgia")
    .sizedByWeight(10,60)
    .withWordPadding(7)
    .withPlacer(Placers.centerClump());
//    .withPlacer(Placers.horizLine());
    wc.drawAll();
    save(filename);
    clear();
    background(255);
  // Save the image of the wordcram
//  cachedImage = get();  
  }
  
/*
 * Make SPL wordcrams from titleWords
 */
  for(int i=0; i<12; i++){
    String filename = new String();
    StringBuffer buff = new StringBuffer();
    buff.append("data/spl"+i+".gif");
    filename = buff.toString();
    
    wc = new WordCram(this).fromTextString(titleWords[i])
    .withStopWords("olympic olympics null") //words to exclude from count
    .withColorer(darkerByWeight(0,68,27))
    .angledAt(radians(0))
    .withFont("Georgia")
    .sizedByWeight(10,60)
    .withWordPadding(7)
    .withPlacer(Placers.centerClump());
//    .withPlacer(Placers.horizLine());
    wc.drawAll();
    save(filename);
    clear();
    background(255);

  }
  
}

void draw() {
 
}


WordColorer darkerByWeight(int red, int green, int blue) {
  final int r = red;
  final int g = green;
  final int b = blue;
  return new WordColorer() {
    public color colorFor(Word word) {
      return color(r,g,b,90/pow(1-word.weight,3));
    }
  }; // Don't forget the semi-colon for the return statement.
}
